package com.example.tdl.service;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.*;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.io.Serializable;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

@Service
public class TokenRedisService {
    private static Logger logger = Logger.getLogger(TokenRedisService.class);
    @Autowired
    private RedisTemplate tokenRedisTemplate;

    /**
     * 批量删除对应的value
     *
     * @param keys
     */
    public void remove(final String... keys) {
        for (String key : keys) {
            remove(key);
        }
    }

    /**
     * 哈希 添加
     *
     * @param key
     * @param hashKey
     * @param value
     */

    public void hmSet(String key, Object hashKey, Object value) {
        HashOperations<String, Object, Object> hash = tokenRedisTemplate.opsForHash();
        hash.put(key, hashKey, value);
    }

    /**
     * 哈希获取数据
     *
     * @param key
     * @param hashKey
     * @return
     */

    public Object hmGet(String key,Object hashKey) {
        try {
            HashOperations<Serializable,Object,Object> hash = tokenRedisTemplate.opsForHash();
            return hash.get(key,hashKey);
        }catch (Exception e){
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 列表添加
     *
     * @param k
     * @param v
     */

    public void Push(String k, Object v) {
        ListOperations<String, Object> list = tokenRedisTemplate.opsForList();
        list.rightPush(k, v);
    }

    /**
     * 列表获取
     *
     * @param k
     * @param l
     * @param l1
     * @return
     */

    public List<Object> Range(String k, long l, long l1) {
        ListOperations<String, Object> list = tokenRedisTemplate.opsForList();
        return list.range(k, l, l1);
    }


    public Set<Object> rangeByScore(String key, double scoure, double scoure1) {
        ZSetOperations<String, Object> zset = tokenRedisTemplate.opsForZSet();
        return zset.rangeByScore(key, scoure, scoure1);
    }

    //删除hash中的key
    public void delHashKey(String key, String Hashkey) {
        HashOperations<Serializable, Object, Object> hash = tokenRedisTemplate.opsForHash();
        hash.delete(key,Hashkey);
    }

    public Object getHashKeys(String key) {
        String ret="{error}";

        Object result = ret;
        try {
            HashOperations<Serializable, Object, Object> hash = tokenRedisTemplate.opsForHash();
            result = hash.keys(key);
            return  result;
        }catch (Exception e){
            e.printStackTrace();
        }
        return result;
    }

    public String get(final String key) {
        String result = null;
        try {
            ValueOperations<Serializable, String> operations = tokenRedisTemplate.opsForValue();
            result = operations.get(key);
            return result;
        }catch (Exception e){
            e.printStackTrace();
        }
        return result;
    }

    public void expire(final String key,final Integer seconds) {
        try {
            tokenRedisTemplate.expire(key,seconds, TimeUnit.SECONDS);
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    public void set(final String key,final String value,final Integer seconds){
        try{
            ValueOperations<Serializable, String> operations = tokenRedisTemplate.opsForValue();
            operations.set(key,value,seconds,TimeUnit.SECONDS);
        }catch (Exception e){
            e.printStackTrace();
            logger.info(e.toString());
        }
    }

    public void delKey(final String... key){
        if(key!=null && key.length > 0){
            if(key.length == 1){
                tokenRedisTemplate.delete(key[0]);
            }else{
                tokenRedisTemplate.delete(CollectionUtils.arrayToList(key));
            }
        }
    }

}
