package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.RoleHasPermissionService;
import com.example.tdl.service.TokenRedisService;
import com.example.tdl.service.UserService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;


@RestController
@RequestMapping("/user")
public class UserController {

    private Logger logger = LoggerFactory.getLogger(UserController.class);

    private CommFeedback fb = new CommFeedback();

    private Gson gson = new Gson();

    @Autowired
    private UserService userService;

    @Autowired
    private RoleHasPermissionService roleHasPermissionService;

    @Autowired
    private TokenRedisService tokenRedisService;


    //获取所有用户信息
    @ApiOperation(value = "获取所有用户信息",notes = "获取所有用户信息")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(HttpServletRequest request){
        UserRedisVo user =  gson.fromJson(tokenRedisService.get("TOKEN_" +request.getHeader("Account_token")),UserRedisVo.class);
        List<String> permission=roleHasPermissionService.getByRoleName(user.getRoleName());
        if (!permission.contains("用户列表")){
            fb.setCode(0);
            fb.setMessage("对不起，您没有此权限");
            return gson.toJson(fb);
        }else{
            return userService.getAll();
        }
    }

    //根据条件查询
    @ApiOperation(value = "根据条件查询",notes = "根据条件查询")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/getByTerm",method = RequestMethod.POST)
    public Object getByTerm(@RequestBody UserTermVo userTermVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        List<String> permission=roleHasPermissionService.getByRoleName(user.getRoleName());
        if (!permission.contains("用户列表")){
            fb.setCode(0);
            fb.setMessage("对不起，您没有此权限");
            return gson.toJson(fb);
        }else{
            return userService.getByTerm(userTermVo);
        }
    }

    //新增用户
    @ApiOperation(value = "新增用户",notes = "新增用户都要传:" +
            "   userName：用户名，" +
            "   password：密码，" +
            "   nickName：昵称，" +
            "   phone：电话，" +
            "   email：邮件，" +
            "   roleName：角色名")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/addUser",method = RequestMethod.POST)
    public Object addUser(@RequestBody AddUserVo userVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        List<String> permission=roleHasPermissionService.getByRoleName(user.getRoleName());
        if (!permission.contains("用户列表")){
            fb.setCode(0);
            fb.setMessage("对不起，您没有此权限");
            return gson.toJson(fb);
        }else{
            if (StringUtils.isEmpty(userVo.getUserName())){
                fb.setCode(0);
                fb.setMessage("用户名不能为空");
                return gson.toJson(fb);
            }
            if (!userVo.getUserName().matches("^[a-zA-Z][a-zA-Z0-9_]{5,19}$")) {
                fb.setCode(0);
                fb.setMessage("用户名以字母开头,可包含数字和_,6-20个字符");
                return gson.toJson(fb);
            }
            if(userService.getByUserName(userVo.getUserName()) !=null){
                fb.setCode(0);
                fb.setMessage("用户名已被占用");
                return gson.toJson(fb);
            }
            if(StringUtils.isEmpty(userVo.getPassword())){
                fb.setCode(0);
                fb.setMessage("密码不能为空");
                return gson.toJson(fb);
            }
            if(!userVo.getPassword().matches("^[a-zA-Z\\d_]{6,20}$")){
                fb.setCode(0);
                fb.setMessage("密码必须是6-20位的数字、字母和_");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(userVo.getNickName())){
                fb.setCode(0);
                fb.setMessage("昵称不能为空");
                return gson.toJson(fb);
            }
            if(StringUtils.isEmpty(userVo.getPhone())){
                fb.setCode(0);
                fb.setMessage("电话不能为空");
                return gson.toJson(fb);
            }
            if( !userVo.getPhone().matches("^1[34578]\\d{9}$")){
                fb.setCode(0);
                fb.setMessage("手机号不规范");
                return gson.toJson(fb);
            }
            if ((userService.getByPhone(userVo.getPhone()))!=null){
                fb.setCode(0);
                fb.setMessage("该手机号已被注册");
                return gson.toJson(fb);
            }
            if(StringUtils.isEmpty(userVo.getEmail())){
                fb.setCode(0);
                fb.setMessage("邮箱不能为空");
                return gson.toJson(fb);
            }
            if( !userVo.getEmail().matches("\\w+(\\.\\w*)*@\\w+(\\.\\w{2,3}){1,3}")){
                fb.setCode(0);
                fb.setMessage("邮箱格式不规范");
                return gson.toJson(fb);
            }
            if ((userService.getByEmail(userVo.getEmail()))!=null){
                fb.setCode(0);
                fb.setMessage("该邮箱已被注册");
                return gson.toJson(fb);
            }
            if (StringUtils.isEmpty(userVo.getRoleName())){
                fb.setCode(0);
                fb.setMessage("角色名不能为空");
                return gson.toJson(fb);
            }
            int a=userService.addUser(userVo);
            if (a>0){
                fb.setCode(1);
                fb.setMessage("添加用户成功");
            }else{
                fb.setCode(0);
                fb.setMessage("添加用户失败");
            }
        }
        return gson.toJson(fb);
    }

    //修改用户
    @ApiOperation(value = "修改用户",notes = "修改用户都要传" +
            "   userNumber：用户编号，" +
            "   nickName：昵称，" +
            "   phone：电话，" +
            "   email：邮件，" +
            "   roleName：角色名")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/updateUser",method = RequestMethod.POST)
    public Object updateUser(@RequestBody UpdateUserVo userVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        List<String> permission=roleHasPermissionService.getByRoleName(user.getRoleName());
        if (!permission.contains("用户列表")){
            fb.setCode(0);
            fb.setMessage("对不起，您没有此权限");
            return gson.toJson(fb);
        }
        ResultUserVo userInfo=userService.getByUserNumber(user.getUserNumber());
//            if (userVo.getUserName().equals("")||userVo.getUserName()==null){
//                fb.setCode(0);
//                fb.setMessage("用户名不能为空");
//            }
//            if (!userVo.getUserName().matches("^[a-zA-Z][a-zA-Z0-9_]{5,19}$")) {
//                fb.setCode(0);
//                fb.setMessage("用户名以字母开头,可包含数字和_,6-20个字符");
//                return gson.toJson(fb);
//            }
//            if( userInfo!=null&&!userInfo.getUserName().equals(userVo.getUserName())){
//                fb.setCode(0);
//                fb.setMessage("用户名已被占用");
//                return gson.toJson(fb);
//            }
//            if(StringUtils.isEmpty(userVo.getPassword())){
//                fb.setCode(0);
//                fb.setMessage("密码不能为空");
//                return gson.toJson(fb);
//            }
//            if(!userVo.getPassword().matches("^[a-zA-Z\\d_]{6,20}$")){
//                fb.setCode(0);
//                fb.setMessage("密码必须是6-20位的数字、字母和_");
//                return gson.toJson(fb);
//            }
        if (StringUtils.isEmpty(userVo.getNickName())){
            fb.setCode(0);
            fb.setMessage("昵称不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(userVo.getPhone())){
            fb.setCode(0);
            fb.setMessage("电话不能为空");
            return gson.toJson(fb);
        }
        if( !userVo.getPhone().matches("^1[34578]\\d{9}$")){
            fb.setCode(0);
            fb.setMessage("手机号不规范");
            return gson.toJson(fb);
        }
        if (userInfo!=null&&userInfo.getPhone().equals(userVo.getPhone())){
            fb.setCode(0);
            fb.setMessage("该手机号已被注册");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(userVo.getEmail())){
            fb.setCode(0);
            fb.setMessage("邮箱不能为空");
            return gson.toJson(fb);
        }
        if( !userVo.getEmail().matches("\\w+(\\.\\w*)*@\\w+(\\.\\w{2,3}){1,3}")){
            fb.setCode(0);
            fb.setMessage("邮箱格式不规范");
            return gson.toJson(fb);
        }
        if (userInfo!=null&&userInfo.getEmail().equals(userVo.getEmail())){
            fb.setCode(0);
            fb.setMessage("该邮箱已被注册");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(userVo.getRoleName())){
            fb.setCode(0);
            fb.setMessage("角色名不能为空");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(userVo.getUserNumber())){
            fb.setCode(0);
            fb.setMessage("需要修改的用户编号不能为空");
            return gson.toJson(fb);
        }
        int a=userService.updateUser(userVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage("修改用户成功");
        }else{
            fb.setCode(0);
            fb.setMessage("修改用户失败");
        }
        return gson.toJson(fb);
    }

    //删除用户
    @ApiOperation(value = "删除用户",notes = "删除用户")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/delUser",method = RequestMethod.POST)
    public Object delUser(@RequestBody String userNumber,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        List<String> permission=roleHasPermissionService.getByRoleName(user.getRoleName());
        if (!permission.contains("用户列表")){
            fb.setCode(0);
            fb.setMessage("对不起，您没有此权限");
            return gson.toJson(fb);
        }
        JSONObject jsonObject= JSON.parseObject(userNumber);
        userNumber=jsonObject.getString("userNumber");
        int a = userService.delUser(userNumber);
        if (a>0){
            fb.setCode(0);
            fb.setMessage("删除用户成功");
        }else{
            fb.setCode(0);
            fb.setMessage("删除用户失败");
        }
        return gson.toJson(fb);
    }

    //重置密码
    @ApiOperation(value = "重置密码",notes = "重置密码")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @RequestMapping(value="/resetPassword",method = RequestMethod.POST)
    public Object resetPassword(@RequestBody String userNumber, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        List<String> permission=roleHasPermissionService.getByRoleName(user.getRoleName());
        if (!permission.contains("用户列表")){
            fb.setCode(0);
            fb.setMessage("对不起，您没有此权限");
            return gson.toJson(fb);
        }
        JSONObject jsonObject= JSON.parseObject(userNumber);
        userNumber=jsonObject.getString("userNumber");
        String pwd=DigestUtils.sha256Hex("Witium123");
        int a = userService.resetPassword(userNumber,pwd);
        if (a>0){
            fb.setCode(0);
            fb.setMessage("重置密码成功");
        }else{
            fb.setCode(0);
            fb.setMessage("重置密码失败");
        }
        return gson.toJson(fb);
    }

    //获取当前的用户名
    @ApiOperation(value = "获取当前的用户名",notes = "获取当前的用户名")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @GetMapping(value="/getUserName")
    public Object getUserName(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        return new UserNameVo(gson.fromJson(datum,UserRedisVo.class).getUserName());
    }

}
